#!/usr/bin/env python3

import objc, html, os
from AppKit import *
from comipoli_view import ComipoliView
from comipoli_archive import ComipoliArchive
from comipoli_toolbar import ComipoliToolbar
import comipoli_numsort

class ComipoliWindow(NSWindow):
    def initWithFrame_(self, frame):
        objc.super(ComipoliWindow, self).initWithContentRect_styleMask_backing_defer_(
            frame,
            NSTitledWindowMask |
            NSClosableWindowMask |
            NSResizableWindowMask |
            NSMiniaturizableWindowMask,
            NSBackingStoreBuffered,
            False)
        # var
        self.is_unrar = False
        self.is_7za = False
        self.num = 0
        self.is_fullscreen = False
        # Event
        self.setDelegate_(self)
        # archive
        self.archive = ComipoliArchive()
        # NSView
        self.canvas = ComipoliView.new()
        self.canvas.setFrameSize_(frame.size)
        self.contentView().addSubview_(self.canvas)
        # Toolbar
        self.toolbar = ComipoliToolbar.new()
        self.toolbar.open_button.setTarget_(self)
        self.setTitleVisibility_(1)
        self.setToolbar_(self.toolbar)
        #
        return self

    @objc.python_method
    def set_path(self, path):
        self.archive.new_archive(path)
        self.change_pixbuf(0)

    @objc.python_method
    def toggle_full_screen(self):
        self.toggleFullScreen_(self)

    def onOpenButtonClicked_(self, button):
        menu = NSApp.mainMenu().itemAtIndex_(2)
        menu.setEnabled_(False)
        self.open_panel = NSOpenPanel.openPanel()
        self.open_panel.setCanChooseDirectories_(True)
        self.open_panel.setAllowsMultipleSelection_(False)
        arr = ["cbz","zip"]
        if self.is_unrar:
            arr.append("cbr")
            arr.append("rar")
        if self.is_7za:
            arr.append("cb7")
            arr.append("7z")
        self.open_panel.setAllowedFileTypes_(arr)
        #self.open_panel.beginWithCompletionHandler( (res)=> {
        self.open_panel.beginSheetModalForWindow_completionHandler_(self, self.openCb_)
        menu.setEnabled_(True)

    @objc.python_method
    def change_pixbuf(self, nex):
        length = len(self.archive)
        if length == 0:
            return
        try:
            num = self.num + nex if self.canvas.spread else self.num + nex - 1
            if length < num + 1 and nex != 50:
                path = os.path.dirname(self.archive.path)
                exts = 'cbz'
                if self.is_pdf:
                    exts += '|pdf'
                if self.is_unrar:
                    exts += '|cbr'
                if self.is_7za:
                    exts += '|cb7'
                cbzs = comipoli_numsort.get_file_numsort_list(path, exts)
                l = len(cbzs)
                if l == 0:
                    return
                else:
                    next_index = cbzs.index(os.path.basename(self.archive.path)) + 1
                    '''if l > next_index:
                        esc = html.escape(cbzs[next_index])
                        dlg = Gtk.MessageDialog(
                            transient_for=self,
                            buttons=Gtk.ButtonsType.OK_CANCEL,
                            message_type=Gtk.MessageType.QUESTION,
                            text='<span bgcolor="red">Next:</span> {}'.format(esc),
                            use_markup=True,
                            secondary_text='ESC Cancel')
                        dlg.set_default_response(Gtk.ResponseType.OK)
                        if dlg.run() == Gtk.ResponseType.OK:
                            uri = GLib.filename_to_uri(GLib.build_filenamev([path, cbzs[next_index]]), None)
                            self.set_uri(uri)
                        dlg.destroy()
                    elif self.is_fullscreen:
                        self.toggle_fullscreen()'''
            else:
                if nex == 0:
                    # Start Page
                    self.canvas.first_page = self.archive[0]
                    self.num = 0
                    self.canvas.spread = False
                    self.canvas.display()
                elif nex == 50:
                    # Last Page
                    self.canvas.first_page = self.archive[length - 1]
                    self.num = length - 1
                    self.canvas.spread = False
                    self.canvas.display()
                elif nex > 0:
                    # Next Page
                    if length > num:
                        # Set Right Page
                        self.canvas.first_page = self.archive[num]
                        if self.is_oblong(self.canvas.first_page):
                            self.canvas.spread = False
                        else:
                            if length > num + 1:
                                # Spread View
                                self.canvas.second_page = self.archive[num + 1]
                                if self.is_oblong(self.canvas.second_page):
                                    self.canvas.spread = False
                                else:
                                    self.canvas.spread = True
                            else:
                                self.canvas.spread = False
                        self.num = num
                        self.canvas.display()
                else:
                    # Previos Page
                    num2 = self.num + nex
                    if num2 < 0:
                        self.num = 0
                        self.canvas.first_page = self.archive[0]
                        self.canvas.spread = False
                        self.canvas.display()
                    else:
                        # Check (width > height)
                        pixbuf = self.archive[num2]
                        pixbuf2 = self.archive[num2 + 1] if length > 1 else None
                        if self.is_oblong(pixbuf) or self.is_oblong(pixbuf2):
                            # Single View
                            self.canvas.first_page = pixbuf2
                            self.canvas.spread = False
                            self.num = num2 + 1
                        else:
                            # Spread View
                            self.canvas.first_page = pixbuf
                            self.canvas.second_page = pixbuf2
                            self.canvas.spread = True
                            self.num = num2
                        self.canvas.display()
                # Draw Page Number
                title = os.path.basename(self.archive.path)
                self.toolbar.title_label.setStringValue_('{0} {1}/{2}'.format(title, self.num + 1, length))
        except Exception as e:
            #ComipoliMessage.show(self.app.props.active_window, e)
            pass

    @objc.python_method
    def is_oblong(self, buf):
        if buf == None:
            return True
        if self.archive.status == 3:
            w, h = buf.get_size()
            return w - h > 0
        else:
            return buf.size().width - buf.size().height > 0

    def openCb_(self, res):
        if res == NSFileHandlingPanelOKButton:
            url = self.open_panel.URLs()[0]
            self.set_path(url.path())

    def windowDidResize_(self, sender):
        aw = self.contentView().frame().size.width
        ah = self.contentView().frame().size.height
        size = NSMakeSize(aw, ah)
        self.canvas.setFrameSize_(size)

    def windowWillClose_(self, sender):
        NSApp.terminate_(sender)

    def windowWillEnterFullScreen_(self, sender):
        self.toolbar.setVisible_(False)
        self.is_fullscreen = True

    def windowDidExitFullScreen_(self, sender):
        self.toolbar.setVisible_(True)
        self.is_fullscreen = False
